#if !defined(__ILMXDDE)
#define __ILMXDDE

// Special handling for C++ code
#ifdef __cplusplus
    extern "C" {
#endif  // __cplusplus


/*---------------------------------------------------------------------------
 * Name:    GMDDE.H
 * Purpose: Header file for application calling GMDDE client routines
 * Author:  Ken Dobson, Copyright (c) IntelliLink Corporation, 1993
 *-------------------------------------------------------------------------*/

//----- GMDDE types
#define MX_PSTRING  LPCSTR
#define MX_PVOID    LPVOID
#define MX_CODE     int

//----- Size defines
#define MX_MAX_CITATION         200     // Maximum size of a citation
#define MX_MAX_APPNAME          50      // Maximum size of an application name 
#define MX_MAX_DLLNAME          65      // Maximum size of a dll name
#define MX_MAX_EXPORTCALLBACK   32      // Maximum size of a call back name
#define MX_MAX_IMPORTCALLBACK   32      // Maximum size of a call back name
#define MX_MAX_FILEFILTER       65      // Maximum size of a file filter
#define MX_MAX_FILENAME         65      // Maximum size of a file name
#define MX_MAX_NUMFILES          5      // Maximum number of file names

//----- String constants
#define MX_REG_FILENAME "mxchange.reg"
       
typedef FILENAME[MX_MAX_FILENAME];

//----- Data structures used for package registration
typedef struct
{
  char         szCitation[MX_MAX_CITATION];                 // Buffer for the Citation
  char         szAppName[MX_MAX_APPNAME];                   // Buffer for the Application Name
  char         szDLLName[MX_MAX_DLLNAME];                   // Buffer for the DLLName
  char         szExportCallback[MX_MAX_EXPORTCALLBACK];     // Buffer for the name of the Export Callback
  char         szImportCallback[MX_MAX_IMPORTCALLBACK];     // Buffer for the name of the Import Callback
  char         szFileFilter[MX_MAX_FILEFILTER];             // Buffer for the File Filter
  short        bHasFiles;                                   // Bool for has files
  short        bFetchItemList;                              // Bool for Fetch Item List
  FILENAME     filename[MX_MAX_NUMFILES];                   // Last Five file names
} MX_Package;

typedef MX_Package far *MX_PPackage;

typedef struct
{
    short       nNumPackages;                               // Number of package structures being returned
    MX_Package  Packages[];                                 // Array of packages to return
} MX_Packages;

typedef MX_Packages far *MX_PPackages;

//----- Prototypes for DDE functions in gmdde.lib
#ifndef GMDDE_API
MX_CODE MX_Startup (void);
MX_CODE MX_ShutDown (void);
MX_CODE MX_PostComplete (                       // Notify completion of translation
               MX_CODE nCompletionCode,         // Translator completion code
               MX_PSTRING szError               // Text string to display (if error)
               );
MX_CODE MX_UpdateMeter (                        // Update progress meter during translation
               short nPercentComplete           // Updated completion percentage (0-100)
               );
MX_CODE MX_InstallPackage (                     // Install a package file
               MX_PSTRING lpFileName            // Name of file to install
               );
MX_CODE MX_ReceiveData (                        // Get data from communicator into file
               MX_PSTRING szCitation,           // Package citation
               MX_PSTRING szItemName,           // Item name in package
               MX_PSTRING szFileName,           // File name to receive data
               MX_PVOID szHeader,               // Pointer to header data
               LONG lHeader                     // Size of header data
               );
MX_CODE MX_SendData (                           // Send data to communicator from file
               MX_PSTRING szCitation,           // Package citation
               MX_PSTRING szItemName,           // Item name in package
               MX_PSTRING szFileName,           // File name to receive data
               MX_PVOID szHeader,               // Pointer to header data
               LONG lHeader                     // Size of header data
               );
MX_CODE MX_RegisterPackage (                    // Register new external translator
               MX_PSTRING szCitation,           // Package name on communicator
               MX_PSTRING szAppName,            // PC application name
               MX_PSTRING szDLLName,            // Translator DLL name
               MX_PSTRING szExportCallback,     // Entry point for export operation
               MX_PSTRING szImportCallback,     // Entry point for import operation
               MX_PSTRING szFileFilter,         // File filter (if bHasFiles == TRUE)
               short bHasFiles,                 // Enable file browsing for PC app?
               short bFetchItemList             // Can the package return datanames?
               );
MX_CODE MX_UnregisterPackage (                  // Remove registered package/application combo
               MX_PSTRING szCitation,           // Package name
               MX_PSTRING szAppName             // Application name
               );
MX_CODE MX_RetrievePackageList (                // Get list of registered packages
               MX_PSTRING szCitation,           // Package name (or NULL for all)
               MX_PSTRING szAppName,            // Application name (or NULL for all)
               HANDLE *mx_hPackages             // Return handle to list data
               );
MX_CODE MX_AddFileName (                        // Add filename to list for package/app pair
               MX_PSTRING  szCitation,          // Package name
               MX_PSTRING  szAppName,           // Application name
               MX_PSTRING  szFileName           // File name to add to end of list
               );         
LONG MX_GetDataNames (                          // Return list of datanames
               MX_PSTRING  szCitation           // Package name to fetch list from
               );
#endif

//----- Return Codes
#define MX_OK                    0    // Operation successful
#define MX_FAIL                  1    // Unable to complete operation
#define MX_ERR_CONNECT           2    // Not connected to Magic Cap
#define MX_ERR_FILE              3    // Named file does not exist
#define MX_ERR_SESSION           4    // Session not started
#define MX_ERR_PARAMS            5    // Invalid parameters supplied
#define MX_ERR_ACTIVE            6    // Session already active
#define MX_ERR_NOOP              7    // No operation in progress
#define MX_ERR_DLL               8    // Unable to find named DLL
#define MX_ERR_HIDDEN            9    // Magic Xchange meter is hidden
#define MX_ERR_USERCAN           10   // User cancelled operation
#define MX_ERR_NOMEM             11   // Cannot allocate memory
#define MX_ERR_NOPACKAGE         12   // Package not found
#define MX_ERR_NODATANAME        13   // Dataname not found in package
#define MX_ERR_BADHEADER         14   // Invalid header data
#define MX_ERR_NOFUNCTION        15   // Function not found in DLL
#define MX_ERR_IN_PROGRESS       16   // Other xfer operation in progress
#define MX_ERR_REGISTERED        17   // Package/app pair already registered
#define MX_ERR_CANT_OPEN_REG     18   // Cannot open/create registration file
#define MX_ERR_CLOSE             19   // Error closing reg file
#define MX_ERR_NOT_REGISTERED    20   // Package/app pair not registered

// Special handling for C++ code
#ifdef __cplusplus
}
#endif // __cplusplus

#endif
