// mainfrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "notexfer.h"

#include "dataitem.h"                  // Data Item dialog 
#include "mainfrm.h"
#include "messaged.h"                  // Message dialog 
//----- Local routines
static void reportError (int nErrorRID);
static void fullPath (LPSTR pFullPath);

//----- Local defines
#define NOTECARDXFER_APPL "NoteXfer"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
   //{{AFX_MSG_MAP(CMainFrame)
   ON_WM_CREATE()
   ON_COMMAND(ID_FILE_EXPORT, OnFileExport)
   ON_COMMAND(ID_FILE_IMPORT, OnFileImport)
   ON_COMMAND(ID_FILE_INSTALL, OnFileInstall)
   ON_WM_DESTROY()
   ON_COMMAND(ID_FILE_REGISTER, OnFileRegister)
   ON_COMMAND(ID_FILE_UNREGISTER, OnFileUnregister)
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// arrays of IDs used to initialize control bars

// toolbar buttons - IDs are command buttons
static UINT BASED_CODE buttons[] =
{
   // same order as in the bitmap 'toolbar.bmp'
   ID_FILE_INSTALL,
   ID_FILE_REGISTER,
   ID_FILE_UNREGISTER,
      ID_SEPARATOR,
   ID_FILE_EXPORT,
   ID_FILE_IMPORT,
      ID_SEPARATOR,
   ID_APP_ABOUT,
};

static UINT BASED_CODE indicators[] =
{
   ID_SEPARATOR,           // status line indicator
//   ID_INDICATOR_CAPS,
//   ID_INDICATOR_NUM,
//   ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
   // TODO: add member initialization code here
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
   int nRc;                            // Return code
   
   if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
      return -1;

   if (!m_wndToolBar.Create(this) ||
      !m_wndToolBar.LoadBitmap(IDR_MAINFRAME) ||
      !m_wndToolBar.SetButtons(buttons,
        sizeof(buttons)/sizeof(UINT)))
   {
      TRACE("Failed to create toolbar\n");
      return -1;      // fail to create
   }

   if (!m_wndStatusBar.Create(this) ||
      !m_wndStatusBar.SetIndicators(indicators,
        sizeof(indicators)/sizeof(UINT)))
   {
      TRACE("Failed to create status bar\n");
      return -1;      // fail to create
   }

   //----- Resize window
   RECT rct;
   rct.left    = 100;
   rct.top     = 100;
   rct.right   = rct.left + 400;
   rct.bottom  = rct.top + 200;
   MoveWindow (&rct, TRUE);
   
   //----- Start a session with Magic Xchange
   nRc = MX_Startup();
   if (nRc)
   {
      reportError (IDS_ERR_STARTUP);
      return 0;
   }
   
   return 0;
}


void CMainFrame::OnDestroy()
{
   int nRc;                            // Return code
   
   CFrameWnd::OnDestroy();
   
   //----- Clean up and go home
   nRc = MX_ShutDown();
   if (nRc)
      reportError (IDS_ERR_SHUTDOWN);
      
}
/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
   CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
   CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnFileExport()
{
   int nRc;                            // Return code
   long lParam;                        // Parameter value for MX call
   CString sDataItem;                  // Data item to be sent to
   CString sFileName;                  // Filename containing source data
   CDataItem *pDIDlg;                  // Ptr to dialog box object
     
   //----- Get the data item operand
   pDIDlg = new CDataItem(NULL, 1);
   nRc = pDIDlg-> DoModal();
   sDataItem = pDIDlg-> m_sDataItem;
   delete pDIDlg;
   if (nRc == IDCANCEL)      
      return;
   
   //----- Get filename operand
   static char BASED_CODE szFilter[] = "Text files (*.txt) | *.txt |All Files (*.*) | *.* ||";
   CFileDialog *pFileDlg = new CFileDialog (TRUE, NULL, NULL, 0, szFilter, NULL);
   nRc = pFileDlg-> DoModal();
      
   sFileName = pFileDlg-> GetPathName();
   delete pFileDlg;
   if (nRc == IDCANCEL)      
      return;
   
   //----- Attempt to export file to notecard via Magic Xchange
   lParam = -1;
   nRc = MX_SendData (NOTECARDXFER_PACKAGE, sDataItem, sFileName, &lParam, sizeof (lParam));
   if (nRc != MX_OK)
   {
      //----- Convert return code to resource ID
      switch (nRc)
      {
         case MX_ERR_CONNECT:
            nRc = IDS_ERR_CONNECTED;
            break;
         case MX_ERR_FILE:
            nRc = IDS_ERR_EXPORT_FILE;
            break;
         case MX_ERR_NOFUNCTION:
            nRc = IDS_ERR_NOT_IMPLEMENTED;
            break;
         case MX_ERR_NOPACKAGE:
            nRc = IDS_ERR_PACKAGE_NOTFOUND;
            break;
         default:
            nRc = IDS_ERR_EXPORT;
      }
      
      //----- Tell the user what happened
      reportError (nRc);
   }
   
   //----- Restore focus here
   SetFocus();
      
   return;   
}

void CMainFrame::OnFileImport()
{
   int nRc;                            // Return code
   long lParam;                        // Parameter value for MX call
   CString sDataItem;                  // Data item to be sent to
   CString sFileName;                  // Filename containing source data
   CDataItem *pDIDlg;                  // Ptr to dialog box object
   
   //----- Get the data item operand
   pDIDlg = new CDataItem(NULL, 0);
   nRc = pDIDlg-> DoModal();
   sDataItem = pDIDlg-> m_sDataItem;
   delete pDIDlg;
   if (nRc == IDCANCEL)      
      return;
   
   //----- Get filename operand
   static char BASED_CODE szFilter[] = "Text files (*.txt) | *.txt |All Files (*.*) | *.* ||";
   CFileDialog *pFileDlg = new CFileDialog (FALSE, NULL, NULL, 0, szFilter, NULL);
   nRc = pFileDlg-> DoModal();
   sFileName = pFileDlg-> GetPathName();
   delete pFileDlg;
   if (nRc == IDCANCEL)      
      return;
      
   //----- Attempt to import notecard to file via Magic Xchange
   lParam = -1;
   nRc = MX_ReceiveData (NOTECARDXFER_PACKAGE, sDataItem, sFileName, &lParam, sizeof (lParam));
   if (nRc != MX_OK)
   {
      //----- Convert return code to resource ID
      switch (nRc)
      {
         case MX_ERR_CONNECT:
            nRc = IDS_ERR_CONNECTED;
            break;
         case MX_ERR_FILE:
            nRc = IDS_ERR_IMPORT_FILE;
            break;
         case MX_ERR_NODATANAME:
            nRc = IDS_ERR_DATANAME_NOTFOUND;
            break;
         case MX_ERR_NOFUNCTION:
            nRc = IDS_ERR_NOT_IMPLEMENTED;
            break;
         case MX_ERR_NOPACKAGE:
            nRc = IDS_ERR_PACKAGE_NOTFOUND;
            break;
         default:
            nRc = IDS_ERR_IMPORT;
      }
      
      //----- Tell the user what happened
      reportError (nRc);
   }
   
   //----- Restore focus here
   SetFocus();
      
   return;   
}

void CMainFrame::OnFileInstall()
{
   int nRc;                            // Return code
   char szWorkFile[_MAX_PATH];         // Filename work area

   //----- Attempt to install package via Magic Xchange
   strcpy (szWorkFile, "ncxfer.pkg");
   fullPath (szWorkFile);
   nRc = MX_InstallPackage (szWorkFile);
   if (nRc != MX_OK)
   {
      //----- Convert return code to resource ID
      switch (nRc)
      {
         case MX_ERR_CONNECT:
            nRc = IDS_ERR_CONNECTED;
            break;
         case MX_ERR_FILE:
            nRc = IDS_ERR_INSTALL_FILE;
            break;
         default:
            nRc = IDS_ERR_INSTALL;
      }
      
      //----- Tell the user what happened
      reportError (nRc);
   }
   
   //----- Restore focus here
   SetFocus();
      
   return;   
}


void CMainFrame::OnFileRegister()
{
   int nRc;                            // Return code
   char szWorkFile[_MAX_PATH];         // Filename work area

   //----- Attempt to register package translator with Magic Xchange
   strcpy (szWorkFile, "notexfer.dll");
   fullPath (szWorkFile);
   nRc = MX_RegisterPackage ( NOTECARDXFER_PACKAGE,
                              NOTECARDXFER_APPL,
                              szWorkFile,
                              "Xlate_GMExport",
                              "Xlate_GMImport",
                              "*.txt",
                              TRUE,
                              TRUE );
   if (nRc != MX_OK)
   {
      //----- Convert return code to resource ID
      switch (nRc)
      {
         case MX_ERR_REGISTERED:
            nRc = IDS_ERR_REGISTERED;
            break;
         case MX_ERR_CANT_OPEN_REG:
            nRc = IDS_ERR_OPEN_REG_FILE;
            break;
         case MX_ERR_CLOSE:
            nRc = IDS_ERR_CLOSE_REG_FILE;
            break;
         default:
            nRc = IDS_ERR_REGISTER;
      }
      
      //----- Tell the user what happened
      reportError (nRc);
   }
   
   //----- Restore focus here
   SetFocus();
      
   return;   
}


//----- Local error reporting routine
static void reportError (int nErrorRID)
{
   //----- Show it to the user
   CMessageDialog messageDlg (nErrorRID);
   messageDlg.DoModal();
   
   return;
}


static void fullPath (LPSTR pFullPath)
{
   char szFname[_MAX_FNAME+_MAX_EXT];  // Input filename work area
   char szWorkFile[_MAX_PATH];         // Full path work area
   char szDrive[_MAX_DRIVE];           // Drive (from split/makepath)
   char szDir[_MAX_DIR];               // Dir (from split/makepath)
   
   //----- Parameter holds filename on input
   _fstrcpy (szFname, pFullPath);
   
   //----- Construct full path here, since our "current" isn't same as Magic Xchange
   GetModuleFileName (AfxGetInstanceHandle(), szWorkFile, sizeof (szWorkFile));
   _splitpath (szWorkFile, szDrive, szDir, NULL, NULL);
   
   //----- Parameter holds filename on input
   _makepath (szWorkFile, szDrive, szDir, szFname, NULL);
   
   //----- Copy result back to caller
   _fstrcpy (pFullPath, szWorkFile);
   
   return;
}
   

void CMainFrame::OnFileUnregister()
{
   int nRc;                            // Return code
   char szWorkFile[_MAX_PATH];         // Filename work area

   //----- Attempt to register package translator with Magic Xchange
   strcpy (szWorkFile, "ncxfer.dll");
   fullPath (szWorkFile);
   nRc = MX_UnregisterPackage (  NOTECARDXFER_PACKAGE,
                                 NOTECARDXFER_APPL);
                              
   if (nRc != MX_OK)
   {
      //----- Convert return code to resource ID
      switch (nRc)
      {
         case MX_ERR_NOT_REGISTERED:
            nRc = IDS_ERR_NOT_REGISTERED;
            break;
         case MX_ERR_CANT_OPEN_REG:
            nRc = IDS_ERR_OPEN_REG_FILE;
            break;
         case MX_ERR_CLOSE:
            nRc = IDS_ERR_CLOSE_REG_FILE;
            break;
         default:
            nRc = IDS_ERR_UNREGISTER;
      }
      
      //----- Tell the user what happened
      reportError (nRc);
   }
   
   //----- Restore focus here
   SetFocus();
      
   return;   
}
