/*--------------------
 * Name:    XLATSTUB.C
 * Purpose: This module contains the registered translator
 *          callback functions for third party translators
 *          which run under the Magic Xchange Third Party API.
 *           
 * Author:  Bill Berthoud, Copyright (c) IntelliLink Corporation, 1994
 *------------------*/

//----- Include Files 
#include <windows.h>
#include <io.h>
#include "gmdde.h"

//----- Global Variables
HANDLE hDLLInstance;            // DLL instance handle

/*-----------------------------------------------------------------------------
 * Name:    LibMain
 * Purpose: DLL initiator function. 
 * Author: Bill Berthoud, Copyright (c) IntelliLink Corporation, 1994
 *---------------------------------------------------------------------------*/
int FAR PASCAL _export LibMain ( HANDLE hInstance,
                         WORD wDataSeg,
                         WORD wHeapSize,
                         LPSTR lpszCmdLine )
{
   //----- This is the usual LibMain stuff
   hDLLInstance = hInstance;
   if (wHeapSize > 0)
      UnlockData (0);

   return 1;
}

/*-----------------------------------------------------------------------------
 * Name:    Xlate_GMImport
 * Purpose: "Copy file" import callback
 * Input:   Itemname, appname, source and target file names
 * Return:  MX_OK or MX_FAIL
 * Author:  Bill Berthoud, Copyright (c) IntelliLink Corporation, 1994
 *---------------------------------------------------------------------------*/
int FAR PASCAL _export Xlate_GMImport (  LPSTR pItemName,
                                         LPSTR pAppName,
                                         LPSTR pFileName,
                                         LPSTR pTempFile )
{
   int rc;                             // Return code
   int hSrc;                           // Source file handle
   int hTar;                           // Target file handle
   long nTotalBytes;                   // Size of file
   long nReadSoFar = 0;                // Bytes read from file so far
   char szBuffer[100];                 // File copy buffer

   //------ Initiate the session
   MX_Startup();

   //------ Open the files
   hSrc = _lopen (pFileName, OF_READ);
   nTotalBytes = filelength (hSrc);
   hTar = _lcreat (pTempFile, 0);

   //----- Loop, reading & writing buffers until all done
   while (rc = _lread (hSrc, szBuffer, sizeof(szBuffer)))
   {
      //----- Bail out if an error occurred
      if (rc == -1)
         break;

      //----- Write to target file
      _lwrite (hTar, szBuffer, rc);

      //----- Update progress meter in Magic Xchange
      nReadSoFar += rc;
      MX_UpdateMeter((int)((nReadSoFar * 100) / nTotalBytes));

      //----- Exit loop if EOF
      if (rc < sizeof (szBuffer))
         break;
   }

   //----- Indicate completion
   if (rc == -1)
      MX_PostComplete (MX_FAIL,"");
   else
      MX_PostComplete(MX_OK,"");

   //----- Close files
   _lclose (hSrc);
   _lclose (hTar);

   //----- End the session
   MX_ShutDown();

   return (rc == -1) ? MX_FAIL : MX_OK;
}

/*-----------------------------------------------------------------------------
 * Name:    Xlate_GMExport
 * Purpose: "Copy file" export callback
 * Input:   Itemname, appname, source and target file names
 * Return:  MX_OK or MX_FAIL
 * Author:  Bill Berthoud, Copyright (c) IntelliLink Corporation, 1994
 *---------------------------------------------------------------------------*/
int FAR PASCAL _export Xlate_GMExport (  LPSTR pItemName,
                                         LPSTR pAppName,
                                         LPSTR pFileName,
                                         LPSTR pTempFile )
{
   int rc;                             // Return code
   int hSrc;                           // Source file handle
   int hTar;                           // Target file handle
   char szBuffer[100];                 // File copy buffer

   //------ Initiate the session
   MX_Startup();

   //------ Open the files
   hSrc = _lopen (pTempFile, OF_READ);
   hTar = _lcreat (pFileName, 0);

   //----- Loop, reading & writing buffers until all done
   while (rc = _lread (hSrc, szBuffer, sizeof(szBuffer)))
   {
      //----- Bail out if an error occurred
      if (rc == -1)
         break;

      //----- Write to target file
      _lwrite (hTar, szBuffer, rc);

      //----- Update progress meter in Magic Xchange
      MX_UpdateMeter(rc);

      //----- Exit loop if EOF
      if (rc < sizeof (szBuffer))
         break;
   }

   //----- Indicate completion
   if (rc == -1)
      MX_PostComplete (MX_FAIL,"");
   else
      MX_PostComplete(MX_OK,"");

   //----- Close files
   _lclose (hSrc);
   _lclose (hTar);

   //----- End the session
   MX_ShutDown();

   return (rc == -1) ? MX_FAIL : MX_OK;
}
